/* (C) 2016-2021 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */
import {
  ComponentRef,
  Compiler,
  Injectable,
  Input,
  Component,
  ViewContainerRef,
  ComponentFactoryResolver, AfterViewInit, OnInit, ViewChild, ElementRef
} from "@angular/core";
import {Deferred, RSUtils, PrinterAction, ServerDialog} from "binding";
import {RSModalViewComponent} from "./modal-view.component";
import {animate, state, style, transition, trigger} from "@angular/animations";

@Injectable()
export class RSModal {
  static defaultView: ViewContainerRef = undefined;

  // private component: ComponentRef<{}> = undefined; // last created component

  constructor(public compiler: Compiler, public resolver: ComponentFactoryResolver) {
  }

  show(compType: any, data?: any, modalType?: any): Promise<any> {
    const deferred: Deferred = new Deferred();
    if (!data) {
      data = {};
    }
    if (typeof (data.closeable) === "undefined") {
      data.closeable = true;
    }
    if (!modalType) {
      modalType = RSModalViewComponent;
    }
    const factory = this.resolver.resolveComponentFactory(modalType);
    // this.compiler.compileComponentAsync(modalType).then((factory) => {
    // this.compiler.compileModuleAndAllComponentsAsync(modalType).then(a => {
    // RSModal.defaultView.clear(); // would delete also other existing modals!
    const component: ComponentRef<{}> = RSModal.defaultView.createComponent(factory, 0, RSModal.defaultView.injector);
    (deferred.promise as any).component = component as any;
    // var component: ComponentRef<{}> = component = RSModal.defaultView.createComponent(a.componentFactories[0]); //, 0, RSModal.defaultView.injector);
    (<RSModalViewComponent>component.instance).component = component;
    (<RSModalViewComponent>component.instance).deferred = deferred;
    (<RSModalViewComponent>component.instance).data = data;
    // (<RSModalView>component.instance).componentModule = compModule;
    (<RSModalViewComponent>component.instance).componentType = compType;
    (<RSModalViewComponent>component.instance).starter = this;
    if (data.modalClass) {
      (<RSModalViewComponent>component.instance).modalClass = data.modalClass;
    }
    if (data.componentCreatedCallback) {
      data.componentCreatedCallback(component);
    }
    // })
    return deferred.promise;
  }

  hide(prom: any) {
    const com: RSModalViewComponent = prom.component.instance as RSModalViewComponent;
    com.cancelIfAllowed();
  }

  confirm(head: string, question: string, options?: any): Promise<any> {
    if (!options) {
      options = {};
    }
    return this.show(RSConfirmModalComponent, RSUtils.extend({head: head, question: question}, options));
  }

  message(head: string, message: string, options?: any): Promise<any> {
    if (!options) {
      options = {};
    }
    return this.show(RSMessageModalComponent, RSUtils.extend({head: head, message: message}, options));
  }

  serverDialog(dialog: ServerDialog): Promise<any> {
    const data: any = {dialog: dialog};
    switch (dialog.dialogType) {
      case 1: // info
        data.modalClass = "modal-info";
        break;
      case 2: // warning
        data.modalClass = "modal-warning";
        break;
      case 3: // error
        data.modalClass = "modal-error";
        break;
      default:
        break;
    }
    const x = this.show(RSServerDialogModalComponent, data, RSModalViewDialogComponent);
    x.then(() => {
    }, () => {
    });
    return x;
  }
}

@Injectable()
export class RSConfirm {
  constructor(public modal: RSModal) {
  }

  confirm(head: string, question: string, options?: any): Promise<any> {
    if (!options) {
      options = {};
    }
    return this.modal.show(RSConfirmModalComponent, RSUtils.extend({head: head, question: question}, options));
  }
}

@Component({
  selector: "rs-confirm-modal",
  template: `
    <rs-modal-head *ngIf="modal.data.head"><div [innerHTML]="modal.data.head"></div></rs-modal-head>
    <rs-modal-content>
        <div [innerHTML]="modal.data.question"></div>
    </rs-modal-content>
    <rs-modal-footer>
      <div style="width:40%">
        <rs-button (hit)="modal.close(true)" [class]="yesType" [class.big]="true" [class.full]="true" style="margin-right:30px"><h3>{{yes}}</h3></rs-button>
      </div>
      <div style="width:40%">
        <rs-button (hit)="modal.cancel()" [class]="noType" [class.big]="true" [class.full]="true"><h3>{{no}}</h3></rs-button>
      </div>
    </rs-modal-footer>
    `
})
export class RSConfirmModalComponent {
  @Input() yes = "<?php _('Yes') ?>";
  @Input() no = "<?php _('No') ?>";
  @Input() yesType = "secondary";
  @Input() noType = "default";

  constructor(public modal: RSModalViewComponent) {
    if (modal.data.yes) {
      this.yes = modal.data.yes;
    }
    if (modal.data.no) {
      this.no = modal.data.no;
    }
    if (modal.data.yesType) {
      this.yesType = modal.data.yesType;
    }
    if (modal.data.noType) {
      this.noType = modal.data.noType;
    }
  }
}


@Component({
  selector: "rs-message-modal",
  template: `
    <rs-modal-head *ngIf="modal.data.head"><div [innerHTML]="modal.data.head"></div></rs-modal-head>
    <rs-modal-content>
        <div [innerHTML]="modal.data.message"></div>
    </rs-modal-content>
    <rs-modal-footer>
        <rs-button (hit)="modal.close(true)" [class]="okType" [class.big]="true"><h3>{{ok}}</h3></rs-button>
    </rs-modal-footer>
    `
})
export class RSMessageModalComponent {
  @Input() ok = "<?php _('Ok') ?>";
  @Input() okType = "secondary";

  constructor(public modal: RSModalViewComponent) {
    if (modal.data.ok) {
      this.ok = modal.data.ok;
    }
    if (modal.data.okType) {
      this.okType = modal.data.okType;
    }
  }
}

@Component({
  selector: "rs-modal-view",
  template: `
<div #main [@visibleState]="visState" style="opacity: 0" [ngClass]="'modal modal-dialog '+ modalClass" tabindex="1">
    <div class="box" [@visibleState]="visState" (mousedown)="$event.stopPropagation()"><div class="scroll" rsDrag><div #box></div></div></div>
</div>
`,
  animations: [
    trigger("visibleState", [
      state("hidden", style({
        opacity: 0
      })),
      state("visible", style({
        opacity: 1
      })),
      transition("*=>hidden", animate("200ms")),
      transition("*=>visible", animate("200ms"))
    ])
  ]
})
export class RSModalViewDialogComponent implements AfterViewInit, OnInit {
  component: ComponentRef<{}> = undefined;
  @ViewChild("box", {read: ViewContainerRef, static: true}) view;
  @ViewChild("main", {static: true}) main: ElementRef;
  deferred: Deferred = undefined;
  data: any;
  name: string;
  modalClass = "";
  componentType: any;
  componentModule: any;
  visState = "hidden";
  starter: RSModal = undefined;

  constructor(public compiler: Compiler, public resolver: ComponentFactoryResolver) {
    RSModalViewComponent.count++;
    this.name = "rs-modal-" + RSModalViewComponent.count;
  }

  cancelIfAllowed = () => {
    if (this.data.closeable) {
      this.cancel();
    }
  };

  close(result: any) {
    if (this.component) {
      this.visState = "hidden";
      setTimeout(() => {
        if (this.component) {
          this.component.destroy();
        }
        this.component = undefined;
      }, 300);
    } else {
      this.component = undefined;
    }
    if (this.deferred) {
      this.deferred.resolve(result);
    }
  }

  cancel() {
    console.log("cancelModal", this, this.deferred);
    if (this.component) {
      this.visState = "hidden";
      setTimeout(() => {
        if (this.component) {
          this.component.destroy();
        }
        this.component = undefined;
      }, 300);
    } else {
      this.component = undefined;
    }
    if (this.deferred) {
      console.log("reject");
      this.deferred.reject();
      console.log("reject2");
    }
  }

  ngOnInit() {
    this.view.createComponent(this.resolver.resolveComponentFactory(this.componentType));
    setTimeout(() => {
      this.visState = "visible";
    }, 10);
  }

  ngAfterViewInit() {
    this.main.nativeElement.focus();
  }
}

@Component({
  selector: "rs-server-dialog-modal",
  template: `
    <rs-modal-head *ngIf="modal.data.dialog.title"><div [innerHTML]="modal.data.dialog.title"></div></rs-modal-head>
    <rs-modal-content>
        <div [innerHTML]="modal.data.dialog.message"></div>
        <div *ngIf="modal.data.dialog.dialogType===0" style="margin-top:10px;font-size:70%">Note: When printer is blocking and communication buffers are full answers might not be executed!</div>
    </rs-modal-content>
    <rs-modal-footer>
        <rs-button *ngFor="let c of modal.data.dialog.choices;let indexOfelement=index;" (hit)="choice(indexOfelement)" class="secondary big">{{c}}</rs-button>
    </rs-modal-footer>
    `
})
export class RSServerDialogModalComponent {
  @Input() dialog = {};
  icon = "";
  variant = "";

  constructor(public modal: RSModalViewDialogComponent,
              private action: PrinterAction) {
    switch (this.modal.data.dialog.dialogType) {
      case 1: // info
        break;
      case 2: // warning
        break;
      case 3: // error
        break;
      default:
        break;
    }
  }

  choice(idx) {
    this.action.selectDialogChoice(idx, this.modal.data.dialog);
    this.modal.close(true);
  }
}
